/*************************************************************************
 *
 *    Used with ICCARM and AARM.
 *
 *    File name   : gui.c
 *    Description : GUI Routines for Printing Messages
 *
 *    History :
 *    1. Date        : Feb, 2010
 *       Author      : Peter Kampf
 *       Description : Create
 *
 *    $Revision: 14946 $
 **************************************************************************/
#include <stdio.h>
#include <string.h>
#include "hardware.h"
#include "system.h"
#include "scheduler.h"
#include "oled_lib.h"
#include "myfont.h"
#include "chrbig.h"
#include "utils.h"
#include "nmea.h"
#include "gui.h"

//define public pointer to bigfont:
const char *ptrmyfont=myfont;
volatile uint16 g_HeightMax=0;

//----------------------------------------------
//function	: OLED_PrintApplicationDyn()
//out		: NULL
//description	: prints dynamic user data onto OLED
//-----------------------------------------------
extern void OLED_PrintApplicationDyn(void)
{
char acBuffer[C_NMEA_TEMPBUFFERSIZE];
char acBuffer2[C_NMEA_TEMPBUFFERSIZE];
static char *ptrData1, *ptrData2;

  ptrData1=acBuffer;
  switch (g_wdAPPSTAT)
  {
    case App1:
    //PRINT OLED LEFT:
      //TACHO:
      font=myfont;
      sprintf(ptrData1,"%d",pSTC_VTG_phys->SPEEDKMH);
      printfstr(2,0,MAXX,MAXY,XCENTERED+YCENTERED, BORDER,0,ptrData1,0);
      
      //KURS:
      font=chrbig;
      ptrData2=acBuffer2;
      GetCourseSymbol(pSTC_VTG_phys->COURSEDEG_T,&acBuffer2[0]);
      sprintf(ptrData1,"    %s    ",ptrData2);
      printfstr(0,0,MAXX,MAXY,XLEFT+YDOWN,BORDER,0,ptrData1,0);
      
    //PRINT OLED RIGHT:      
      //HHE:
      font=myfont;
      sprintf(ptrData1,"%d",pSTC_GGA_phys->HEIGHT);
      printfstr(2,0,MAXX,MAXY,XCENTERED+YCENTERED, BORDER,0,ptrData1,1);

      font=chrbig;
      if(pSTC_GGA_phys->HEIGHT>g_HeightMax) g_HeightMax=pSTC_GGA_phys->HEIGHT;
      if(g_wdSYSSTAT!=GPS_OK) sprintf(ptrData1,"max:  . . . . .  ");
      else sprintf(ptrData1,"max:  %d m  ",g_HeightMax);
      printfstr(2,0,MAXX,MAXY,XCENTERED+YDOWN,BORDER,0,ptrData1,1);

    break;
    case App2:
    //PRINT OLED LEFT:
      //TACHO:
      font=myfont;
      sprintf(ptrData1,"%d",pSTC_VTG_phys->SPEEDKMH);
      printfstr(2,0,MAXX,MAXY,XCENTERED+YCENTERED, BORDER,0,ptrData1,0);

      //KURS:
      font=chrbig;
      ptrData2=acBuffer2;
      GetCourseSymbol(pSTC_VTG_phys->COURSEDEG_T,&acBuffer2[0]);
      sprintf(ptrData1,"    %s    ",ptrData2);
      printfstr(0,0,MAXX,MAXY,XLEFT+YDOWN,BORDER,0,ptrData1,0);

    //PRINT OLED RIGHT:
      //KURS:
      font=myfont;
      sprintf(ptrData1,"%d",pSTC_VTG_phys->COURSEDEG_T);
      printfstr(2,0,MAXX,MAXY,XCENTERED+YCENTERED, BORDER,0,ptrData1,1);
    break;
    case AppX:
    //PRINT OLED LEFT:
      font=chrbig;
      //DATE:
      sprintf(ptrData1,"Dat: %02d.%02d.20%02d  ",
            pSTC_RMC_phys->DAT_year,
            pSTC_RMC_phys->DAT_month,
            pSTC_RMC_phys->DAT_day
            );
      printfstr(0,0,MAXX,MAXY,XLEFT,BORDER,0,ptrData1,0);
      //TIME:
      sprintf(ptrData1,"UTC: %2d:%02d:%02d  ",
            pSTC_RMC_phys->TIME_h,
            pSTC_RMC_phys->TIME_min,
            pSTC_RMC_phys->TIME_sec
            );
      printfstr(0,12,MAXX,MAXY,XLEFT, BORDER,0,ptrData1,0);
      hline(2,29,100,0); 
      //NUM OF SATs:
      sprintf(ptrData1,"SAT in use: %02s ",pSTC_GGA_ascii->SATELLITES);
      printfstr(0,30,MAXX,MAXY,XLEFT, BORDER,0,ptrData1,0);
      if(pSTC_RMC_ascii->STATUS=='A') sprintf(ptrData1,"GPS: VALID       ");
      else sprintf(ptrData1,"GPS: INVALID  ");
      printfstr(0,42,MAXX,MAXY,XLEFT, BORDER,0,ptrData1,0);
      //DEFMODE:
      ptrData2=acBuffer2;
      switch(pSTC_RMC_ascii->DEFMODE2)
      {
        case 'A':
          strcpy(ptrData2,"ACTIVE\0");
        break;
        case 'V':
          strcpy(ptrData2,"VOID\0");
        break;
        default:
          strcpy(ptrData2,"NDEF\0");
        break;
      }
      sprintf(ptrData1,"MODUS: %s",ptrData2);
      printfstr(0,54,MAXX,MAXY,XLEFT, BORDER,0,ptrData1,0);
      //QUALITY:
      ptrData2=acBuffer2;
      switch(pSTC_GGA_ascii->QUALITY)
      {
      case '0':
          strcpy(ptrData2,"VOID\0");
      break;
      case '1':
          strcpy(ptrData2,"GPS   \0");
      break;
      case '2':
          strcpy(ptrData2,"DGPS \0");
      break;
      case '6':
          strcpy(ptrData2,"ESTM\0");
      break;
      default:  
          strcpy(ptrData2,"NDEF\0");
      break;
      }
      strcpy(ptrData1,"QUALITY: ");
      strcat(ptrData1,ptrData2);
      printfstr(0,66,MAXX,MAXY,XLEFT, BORDER,0,ptrData1,0);
      //----------------------------------------------
      //PRINT OLED RIGHT:
      ptrData1=acBuffer;
      ptrData2=acBuffer2;
      font=chrbig;
      //GPS LONGITUDE:
      sprintf(ptrData1,"%c %s  ",
            pSTC_RMC_ascii->LONGPREFIX,
            pSTC_RMC_ascii->LONGVAL
            );
      printfstr(2,0,MAXX,MAXY,XLEFT,BORDER,0,ptrData1,1);
      //TIME:
      //GPS LATITUDE:
      sprintf(ptrData1,"%c %s  ",
            pSTC_RMC_ascii->LATPREFIX,
            pSTC_RMC_ascii->LATVAL
            );
      printfstr(2,12,MAXX,MAXY,XLEFT,BORDER,0,ptrData1,1);
      //GPS FIX:
      switch(pSTC_GSA_ascii->POSDEF)
      {
        case '1':
          strcpy(ptrData2,"NO FIX\0");
        break;
        case '2':
          strcpy(ptrData2,"2D\0");
        break;
        case '3':
          strcpy(ptrData2,"3D\0");
        break;
        case '4':
          strcpy(ptrData2,"VOID\0");
        break;
        default:
          strcpy(ptrData2,"NDEF\0");
        break;
      }
      strcpy(ptrData1,"FIX: ");
      strcat(ptrData1,ptrData2);
      sprintf(ptrData2,"%s      ",ptrData1);
      printfstr(0,26,MAXX,MAXY,XLEFT, BORDER,0,ptrData2,1);

      ptrData1=acBuffer;
      ptrData2=acBuffer2;
      //EXACT HEIGHT:
      if(strlen(pSTC_GGA_ascii->HEIGHT)>5)
        strcpy(ptrData1,"Hprec: -----------\0");
      else sprintf(ptrData1,"Hprec: %s m    ",pSTC_GGA_ascii->HEIGHT);
      printfstr(0,38,MAXX,MAXY,XLEFT, BORDER,0,ptrData1,1);
      //SPEED KNOTS:
      sprintf(ptrData1,"SP1: %s kn  ",pSTC_VTG_ascii->SPEEDKNOTS);
      printfstr(0,54,MAXX,MAXY,XLEFT, BORDER,0,ptrData1,1);
      //SPEED KM/H:
      sprintf(ptrData1,"SP2: %s kmh  ",pSTC_VTG_ascii->SPEEDKMH);
      printfstr(0,66,MAXX,MAXY,XLEFT, BORDER,0,ptrData1,1);
  }
}

//----------------------------------------------
//function	: OLED_PrintStaticMask()
//out		: NULL
//description	: prints static values onto OLED
//                according to set application
//-----------------------------------------------
extern void OLED_PrintStaticMask(void)
{
char acRAM1[20];
char* ptrData1;

  ptrData1=acRAM1;
  //print header/footer info to disp according to button event:
  font=chrbig;
  switch (g_wdAPPSTAT)
  {
    case App1:
      //print header: (disp 0)
      cls(0,0); //clear disp0
      strcpy(ptrData1,"TACHO: (km/h)");
      printfstr(2,0,MAXX,MAXY,XLEFT,BORDER,0,ptrData1,0);

      cls(0,1); //clear disp1
      //print header: (disp 1)
      strcpy(ptrData1,"HOEHE: (m/NN)");
      printfstr(2,0,MAXX,MAXY,XLEFT,BORDER,0,ptrData1,1);
      //print footer: (disp 1)
    break;
    case App2:
      cls(0,1); //clear disp1
      //print header: (disp 1)
      strcpy(ptrData1,"KOMPASS:");
      printfstr(2,0,MAXX,MAXY,XLEFT,BORDER,0,ptrData1,1);
      //print footer: (disp 1)
      strcpy(ptrData1,"grad");
      printfstr(0,0,MAXX,MAXY,XRIGHT+YDOWN,BORDER,0,ptrData1,1);
    break;
    case AppX:
      cls(0,0); //clear disp0
      cls(0,1); //clear disp1
    break;
    default:
    break;
    }
}

//----------------------------------------------
//function	: OLED_PrintNoDataScreen()
//out		: NULL
//description	: prints startup data onto OLED
//-----------------------------------------------
extern void OLED_PrintNoDataScreen(void)
{
char acRAM1[20];
char* ptrData1;

  cls(0,0);
  cls(0,1);
  ptrData1=acRAM1;
  font=chrbig;
  strcpy(ptrData1,"KEINE DATEN!\0");
  printfstr(2,0,MAXX,MAXY,XCENTERED+YCENTERED,BORDER,0,ptrData1,0);
  printfstr(2,0,MAXX,MAXY,XCENTERED+YCENTERED,BORDER,0,ptrData1,1);
}

//----------------------------------------------
//function	: OLED_PrintStartupScreen()
//out		: NULL
//description	: prints startup data onto OLED
//-----------------------------------------------
extern void OLED_PrintStartupScreen(void)
{
char acRAM1[20];
char* ptrData1;

  ptrData1=acRAM1;
  font=chrbig;
  strcpy(ptrData1,"-------------------");
  printfstr(2,0,MAXX,MAXY,XLEFT,BORDER,0,ptrData1,0);
  printfstr(2,5,MAXX,MAXY,XLEFT,BORDER,0,ptrData1,1);
  strcpy(ptrData1,C_HEADERINFO_TITLE);
  printfstr(2,12,MAXX,MAXY,XLEFT,BORDER,0,ptrData1,0);
  strcpy(ptrData1,C_HEADERINFO_AUTHOR);
  printfstr(2,0,MAXX,MAXY,XLEFT+YDOWN,BORDER,0,ptrData1,0);
  strcpy(ptrData1,"-------------------");
  printfstr(2,24,MAXX,MAXY,XLEFT,BORDER,0,ptrData1,0);
  printfstr(2,29,MAXX,MAXY,XLEFT,BORDER,0,ptrData1,1);

  strcpy(ptrData1,C_HEADERINFO_SWRELEASE);
  printfstr(2,0,MAXX,MAXY,XRIGHT+YDOWN,BORDER,0,ptrData1,1);
}

//----------------------------------------------
//function	: GetCourseSymbol()
//out		: void
//description	: prints startup data onto OLED
//-----------------------------------------------
static void GetCourseSymbol(uint16 course, char* ptrDest)
{
uint8 courseID;
const char *ptrROM;
    //f(deg)=(deg*100+1125)/2250 (16 parts/slices) or
    //f(deg)=(deg*100+2250)/4500 (8 parts/slices)
    courseID=(course*100+2250)/4500; //Resolution: 8 Parts within 2*PI
    courseID&=7; //convert the result: 8 to again 0
    ptrROM = &CourseSymbols[courseID*8];
    //padding for smaller strings (i.e.    NORD   )
    if(courseID%2) {}
    else { strcpy(ptrDest,"    ");  ptrDest+=4; }
    strncpy_p(ptrDest,ptrROM,8);
}