/*************************************************************************
 *
 *    Used with ICCARM and AARM.
 *
 *    File name   : hardware.c
 *    Description : HW Routines
 *
 *    History :
 *    1. Date        : Jan, 2010
 *       Author      : Peter Kampf
 *
 *    $Revision: 14946 $
 **************************************************************************/
#include "hardware.h"

/******************************************************************************
 *
 * Function Name: lowLevelInit()
 *
 * Description:
 *    This function starts up the PLL then sets up the GPIO pins before
 *    waiting for the PLL to lock.  It finally engages the PLL and
 *    returns
 *
 * Calling Sequence: 
 *    void
 *
 * Returns:
 *    void
 *
 *****************************************************************************/
void lowLevelInit(void)
{
  //Init PLL:
  PLLCFG_bit.MSEL=3;                 //M=3(4?),P=1(2?) PLLfreq = 235,9296 Mhz, CCLK = 58,9824 Mhz
  PLLCFG_bit.PSEL=1;

  PLLFEED=0xAA;
  PLLFEED=0x55;
  PLLCON_bit.PLLE=1;                 // Enable the PLL
  PLLFEED=0xAA;
  PLLFEED=0x55;

  while(!(PLLSTAT & PLOCK));  // Wait for PLL to lock

  //Init MAM & Flash memory fetch
  MAMCR_bit.MODECTRL=2;
  MAMTIM_bit.CYCLES=4;
  APBDIV_bit.APBDIV=1;   //pclk = cclk

  //DEFAULT: setup the parallel port pining: (as basic operation before starting up)
  IOCLR = PIO_ZERO_BITS;                // clear the ZEROs output
  IOSET = PIO_ONE_BITS;                 // set the ONEs output
  IODIR = PIO_DEFAULT_IODIR_MASK;       // set the output bit direction to INPUT as DEFAULT (first)

  //DEFAULT: preset port pins to GPIO:
  PINSEL0 &= 0x0;               // set I/O Port0(0-15) to GPIO port
  PINSEL1 &= 0x0;               // set I/O Port1(16-31) to GPIO port
  
  //DEFAULT: switch off LED at startup:
  A_LED1_OFF;
}

/*-----------------------------------------------------------------------------*/
