/*************************************************************************
 *
 *    Used with ICCARM and AARM.
 *
 *    File name   : hardware.h
 *    Description : Hardware Header File
 *
 *    History :
 *    1. Date        : Feb, 2010
 *       Author      : Kampf Peter
 *
 *    $Revision: 1
 **************************************************************************/
#ifndef __HARDWARE_H__
#define __HARDWARE_H__

#include <NXP/iolpc2103.h> 
#include <intrinsics.h>
#include "types.h"

// Declarations of functions coded in hardware.c
void lowLevelInit(void);
void Delay(uint32 del);
void Dly100us(long);

// MEMMAP defines
#define MEMMAP_BBLK   0                 // Interrupt Vectors in Boot Block
#define MEMMAP_FLASH  1                 // Interrupt Vectors in Flash
#define MEMMAP_SRAM   2                 // Interrupt Vectors in SRAM

// some handy DEFINES
#ifndef FALSE
#define FALSE               0
#ifndef TRUE
#define TRUE                !FALSE
#endif
#endif

#ifndef RESET
#define RESET               0
#ifndef SET
#define SET                !RESET
#endif
#endif

#ifndef BIT
#define BIT(n)              (1L << (n))
#endif

#ifndef XORLED1
#define XORLED1(bit)        if(bit) A_LED1_OFF; else A_LED1_ON; bit^=1
#endif

//#define NULL      0

// PLL setup values are computed within the LPC include file
// It relies upon the following defines
#define FOSC                (14745000)  // Master Oscillator Freq.
#define PLL_MUL             (4)         // PLL Multiplier
#define CCLK                (FOSC * PLL_MUL) // CPU Clock Freq.

// Pheripheral Bus Speed Divider
#define PBSD                4           // MUST BE 1, 2, or 4
#define PCLK                (CCLK / PBSD) // Peripheral Bus Clock Freq.

// Do some value range testing
#if ((FOSC < 10000000) || (FOSC > 25000000))
#error Fosc out of range (10MHz-25MHz)
#error correct and recompile
#endif

#if ((CCLK < 10000000) || (CCLK > 60000000))
#error cclk out of range (10MHz-60MHz)
#error correct PLL_MUL and recompile
#endif

#if ((PBSD != 1) && (PBSD != 2) && (PBSD != 4))
#error Pheripheal Bus Speed Divider (PBSD) illegal value (1, 2, or 4)
#endif
/*-------------------------------------------------------------------------------------------------------------------------------*/
//Generic Macros
#define MSB(word)       ((uint8)((word)>>8))              /* used to strip MSB and LSB bytes from 16-bit Words */
#define LSB(word)       ((uint8)(word))
/*-------------------------------------------------------------------------------------------------------------------------------*/
#define SETB(n)	        IOSET |= (n)
#define CLRB(n)	        IOCLR |= (n)

//Hardware Specific Macros -Note the use of structures from IAR Header files
//Actions (Prefix A_)
#define   A_LED1_ON     IOCLR_bit.P0_26=1
#define   A_LED1_OFF    IOSET_bit.P0_26=1
#define   A_LED2_ON     IOCLR_bit.P0_13=1
#define   A_LED2_OFF    IOSET_bit.P0_13=1

#define   OSCILLATOR_CLOCK_FREQUENCY  14745600  //in MHz
#define   PLOCK           0x0400
#define   SPIF            7 //for SPI Status Reg Mask


//I/Os are all switched as DEFAULT input @startup:
#define PIO_DEFAULT_IODIR_MASK           0

// The following are for the OLED/LCD @LPC-P2103 Header Board
// Port Bit Definitions & Macros:    Description - initial conditions
#define OLED_CS1            BIT(0)      // OLED Serial Data Input
#define OLED_CS2            BIT(1)      // OLED Chip Select
#define OLED_DC             BIT(2)      // OLED 
#define OLED_RES            BIT(3)      // OLED Reset
#define OLED_SPI_SCK        BIT(4)      // OLED SPI Serial Clock
#define OLED_SPI_MISO       BIT(5)      // OLED SPI MISO
#define OLED_SPI_MOSI       BIT(6)      // OLED SPI MOSI
#define P07_UNUSED_BIT      BIT(7)      // 
#define TXD_UART1           BIT(8)      // P0.08 TXD UART1 low output
#define RXD_UART1           BIT(9)      // P0.09 RXD UART1 - input
#define P10_UNUSED_BIT      BIT(10)     // 
#define OLED_12V            BIT(11)     // OLED +12V Mains
#define P12_UNUSED_BIT      BIT(12)     // P0.12 LED1 low active
#define P13_UNUSED_BIT      BIT(13)     // P0.13 LED2 low active
#define P14_UNUSED_BIT      BIT(14)     // P0.14 unused - low output
#define P15_UNUSED_BIT      BIT(15)     // P0.15 unused - low output
#define P16_BUTTON_1        BIT(16)     // P0.16 button1 - low output
#define P17_UNUSED_BIT      BIT(17)     // P0.17 unused - low output
#define P18_UNUSED_BIT      BIT(18)     // P0.18 unused - low output
#define P19_UNUSED_BIT      BIT(19)     // P0.19 unused - low output
#define P20_UNUSED_BIT      BIT(20)     // P0.20 unused - low output
#define P21_UNUSED_BIT      BIT(21)     // P0.21 unused - low output
#define P22_UNUSED_BIT      BIT(22)     // P0.22 unused - low output
#define P23_UNUSED_BIT      BIT(23)     // P0.23 unused - low output
#define P24_UNUSED_BIT      BIT(24)     // P0.24 unused - low output
#define P25_UNUSED_BIT      BIT(25)     // P0.25 unused - low output
#define P26_UNUSED_BIT      BIT(26)     // P0.26 unused - low output
#define P27_UNUSED_BIT      BIT(27)     // P0.27 unused - low output
#define P28_UNUSED_BIT      BIT(28)     // P0.28 unused - low output
#define P29_UNUSED_BIT      BIT(29)     // P0.29 unused - low output
#define P30_UNUSED_BIT      BIT(30)     // P0.30 unused - low output
#define P31_UNUSED_BIT      BIT(31)     // P0.31 unused - low output

/*-----------------------------------------------------------------------------*/
/*
Stages for primar defaultconfig: (can be altered by subsequent initialize parts)
1.) Define Pinning
2.) Define potential voltage HIGH or LOW state
3.) Define EVERY pin as input (per DEFAULT) -this avoids potential collisions
*/

/*-----------------------------------------------------------------------------*/
//Define specific outputs as ZERO (LOW) as DEFAULT:
#define PIO_ZERO_BITS       (uint32) ( \
                                         OLED_CS1 | \
                                         OLED_CS2  | \
                                         OLED_DC   | \
                                         OLED_RES  | \
                                         OLED_SPI_SCK   | \
                                         OLED_SPI_MISO  | \
                                         OLED_SPI_MOSI  | \
                                         P07_UNUSED_BIT | \
					 P10_UNUSED_BIT | \
					 OLED_12V       | \
					 P12_UNUSED_BIT | \
					 P13_UNUSED_BIT | \
					 P14_UNUSED_BIT | \
					 P15_UNUSED_BIT | \
				         P22_UNUSED_BIT | \
                                         P23_UNUSED_BIT | \
                                         P24_UNUSED_BIT | \
                                         P25_UNUSED_BIT | \
                                         P26_UNUSED_BIT | \
                                         P27_UNUSED_BIT | \
                                         P28_UNUSED_BIT | \
                                         P29_UNUSED_BIT | \
                                         P30_UNUSED_BIT | \
					 P31_UNUSED_BIT | \
                                         0 )

#define PIO_ONE_BITS        (uint32)     ( \
                                         TXD_UART1      | \
                                         RXD_UART1 | \
                                         P16_BUTTON_1 | \
                                         0 )

/*-----------------------------------------------------------------------------*/
// VIC Channel Assignments
#define VIC_WDT         0
#define VIC_TIMER0      4
#define VIC_TIMER1      5
#define VIC_UART0       6
#define VIC_UART1       7
#define VIC_PWM         8
#define VIC_PWM0        8
#define VIC_I2C         9
#define VIC_SPI         10
#define VIC_SPI0        10
#define VIC_SPI1        11
#define VIC_PLL         12
#define VIC_RTC         13
#define VIC_EINT0       14
#define VIC_EINT1       15
#define VIC_EINT2       16
#define VIC_EINT3       17
#define VIC_ADC         18

// Vector Control Register bit definitions
#define VIC_ENABLE      (1 << 5)

// Convert Channel Number to Bit Value
#define VIC_BIT(chan)   (1 << (chan))

#endif /* __HARDWARE_H__ */

