/*************************************************************************
 *
 *    Used with ICCARM and AARM.
 *
 *    File name   : main.c
 *    Description : Main Routines
 *
 *    History :
 *    1. Date        : Feb 27, 2010
 *       Author      : Peter Kampf
 *
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 * Global System Init // Startup Sequence: 
 * => LowLevelInit: 
 *      - PLL Initialization
 *      - MAM & Flash Init
 *      - Default DIR/SET/CLR setup for I/O's, All Pins=Inputs
 *      - All Pins per Default at Startup: GPIO
 * => SystemInit:
 *      - Map Interrupt Vectors FLASH/SRAM
 *      - Init Interrupts
 *      - Define TIMER/UART/... Int Sources
 *      - Define specific DIR/SET/CLR config
 *      - Define SPI Regs
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 * => OLED Init:
 *      - Issue RESET @ OLED
 *  Commands from table:
 *      - Set Column Address
 *      - Set Row Address
 *      - Set Current Range
 *      - Set Contrast Control
 *      - Set VCOMH Voltage
 *      - Set Precharge Voltage
 *      - Set Re Map
 *      - Set Entire Display OFF
 *      - Set Multiplex Ratio
 *      - Set Phase Length
 *      - Set Row Period
 *      - Set Pre-charge Compensation
 *      - Set Display Clock
 *      - Set Segment Low Voltage (VSL)
 *      - Set Gray Scale Table
 *      - Set Display ON/OFF
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *    $Revision: 14946 $
 **************************************************************************/

#define   ROM_RUN // Interrupt Vectors in FLASH

/******************************************************************************
 * Function Name: main()
 *
 * Description:
 *    This function is the main program
*    void
 * Returns:
 *    int
 *****************************************************************************/

#include "hardware.h"
#include "system.h"
#include "oled_lib.h"
#include "scheduler.h"
#include "init.h"

// *************************************************************************
// /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\ MAIN /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
// *************************************************************************
int main(void)
{
  Mains12V_Off();
  uC_Init();
  OLED_Init();
  Global_Init();
  Mains12V_On();
  EN_Interrupt;
  while (1)  
  {
    sm_process();     // call statemachine/dispacher
  };
}


