/*************************************************************************
 *
 *    Used with ICCARM and AARM.
 *
 *    (c) Copyright IAR Systems 2010
 *
 *    File name   : nmea.c
 *    Description : NMEA Function Library
 *
 *    History :
 *    1. Date        : Feb, 2010
 *       Author      : Peter Kampf
 *
 *    $Revision: 14946 $
 **************************************************************************/
#include <string.h>
#include <stdio.h>
#include <stdbool.h>
#include <stdlib.h>
#include "hardware.h"
#include "system.h"
#include "utils.h"
#include "nmea.h"

//**************************************************************************
//define global variables & structpointers:
//**************************************************************************
STC_NMEA_Buf_RAW NMEABuf_RAW; 
STC_NMEA_Buf_RAW *pSTC_NMEABuf_RAW = &NMEABuf_RAW; // get pointer to struct
//Def. Struct of extracted ASCII Data @GPRMC:
STC_NMEA_RMC_ascii NMEA_RMC_ascii;
STC_NMEA_RMC_ascii *pSTC_RMC_ascii = &NMEA_RMC_ascii;
STC_NMEA_RMC_phys NMEA_RMC_phys;
STC_NMEA_RMC_phys *pSTC_RMC_phys = &NMEA_RMC_phys;
//Def. Struct of extracted ASCII Data @GPGGA:
STC_NMEA_GGA_ascii NMEA_GGA_ascii;
STC_NMEA_GGA_ascii *pSTC_GGA_ascii = &NMEA_GGA_ascii;
STC_NMEA_GGA_phys NMEA_GGA_phys;
STC_NMEA_GGA_phys *pSTC_GGA_phys = &NMEA_GGA_phys;
//Def. Struct of extracted ASCII Data @GPVTG:
STC_NMEA_VTG_ascii NMEA_VTG_ascii;
STC_NMEA_VTG_ascii *pSTC_VTG_ascii = &NMEA_VTG_ascii;
STC_NMEA_VTG_phys NMEA_VTG_phys;
STC_NMEA_VTG_phys *pSTC_VTG_phys = &NMEA_VTG_phys;
//Def. Struct of extracted ASCII Data @GPGSA:
STC_NMEA_GSA_ascii NMEA_GSA_ascii;
STC_NMEA_GSA_ascii *pSTC_GSA_ascii = &NMEA_GSA_ascii;

//--------------------------------------------------------------------------
//function	: GetRMCdataSTC()
//out		: NULL
//description	: Copies RMC_Data from RAW_ASCII to formatted STC_ASCII
//--------------------------------------------------------------------------
extern char* GetRMCdataSTC(char* pNMEA)
{
char acBuffer[C_NMEA_MAXSTRSIZE];
uint8 format_cnt;

    format_cnt=0;
    do
    {
      switch (format_cnt)
      {
        case POS_RMC_GPRMC:
        break;
        case POS_RMC_TIME:
          strextr(pSTC_RMC_ascii->TIME,pNMEA);
        break;
        case POS_RMC_STATUS:
          pSTC_RMC_ascii->STATUS=*pNMEA;
        break;
        case POS_RMC_LATTITUDE:
            strextr(pSTC_RMC_ascii->LATVAL,pNMEA);
        break;
        case POS_RMC_LATPREFIX:
          pSTC_RMC_ascii->LATPREFIX=*pNMEA;
        break;
        case POS_RMC_LONGITUDE:
          strextr(pSTC_RMC_ascii->LONGVAL,pNMEA);
        break;
        case POS_RMC_LONGPREFIX:
          pSTC_RMC_ascii->LONGPREFIX=*pNMEA;
        break;
        case POS_RMC_SPEEDKNOTS:
          strextr(pSTC_RMC_ascii->SPEEDKNOTS,pNMEA);
        break;
        case POS_RMC_COURSEDEG:
          strextr(pSTC_RMC_ascii->COURSEDEG,pNMEA);
        break;
        case POS_RMC_DATE:
          strextr(pSTC_RMC_ascii->DATE,pNMEA);
        break;
        case POS_RMC_DEFMODE2:
          pSTC_RMC_ascii->DEFMODE2=*pNMEA;
        break;
        default:
        break;
      }        
      pNMEA=strextr(acBuffer,pNMEA);
      format_cnt++;
    } while (*pNMEA!=C_NMEA_CKS_EXPECT_CHAR&&(format_cnt<C_NMEA_MAXPACKETSIZE));
    if (format_cnt>=C_NMEA_MAXPACKETSIZE) return NULL;
    //get checksum at the end of transmission:
    strncpy_t(pSTC_RMC_ascii->CKS,++pNMEA,sizeof(pSTC_RMC_ascii->CKS)-1);
return pNMEA;
}

//--------------------------------------------------------------------------
//function	: GetGGAdataSTC()
//out		: NULL
//description	: Copies GGA_Data from RAW_ASCII to formatted STC_ASCII
//--------------------------------------------------------------------------
extern char* GetGGAdataSTC(char* pNMEA)
{
char acBuffer[C_NMEA_MAXSTRSIZE];
uint8 format_cnt;

format_cnt=0;
    do
    {
      switch (format_cnt)
      {
        case POS_GGA_GPGGA:
        break;
        case POS_GGA_QUALITY:
          pSTC_GGA_ascii->QUALITY=*pNMEA;
        break;
        case POS_GGA_SATELLITES:
          strextr(pSTC_GGA_ascii->SATELLITES,pNMEA);
        break;
        case POS_GGA_HDOP:
          strextr(pSTC_GGA_ascii->HDOP,pNMEA);
        break;
        case POS_GGA_HEIGHT:
          strextr(pSTC_GGA_ascii->HEIGHT,pNMEA);
        break;
        default:
        break;
      }        
      pNMEA=strextr(acBuffer,pNMEA);
      format_cnt++;
    } while (*pNMEA!=C_NMEA_CKS_EXPECT_CHAR&&(format_cnt<C_NMEA_MAXPACKETSIZE));
    if (format_cnt>=C_NMEA_MAXPACKETSIZE) return NULL;
    //get checksum at the end of transmission:
    strncpy_t(pSTC_GGA_ascii->CKS,++pNMEA,sizeof(pSTC_GGA_ascii->CKS)-1);
return pNMEA;
}

//--------------------------------------------------------------------------
//function	: GetVTGdataSTC()
//out		: NULL
//description	: Copies VTG_Data from RAW_ASCII to formatted STC_ASCII
//--------------------------------------------------------------------------
extern char* GetVTGdataSTC(char* pNMEA)
{
char acBuffer[C_NMEA_MAXSTRSIZE];
uint8 format_cnt;

format_cnt=0;
    do
    {
      switch (format_cnt)
      {
        case POS_VTG_GPVTG:
        break;
        case POS_VTG_COURSEDEG_T:
          strextr(pSTC_VTG_ascii->COURSEDEG_T,pNMEA);
        break;
        case POS_VTG_COURSEDEG_M:
          strextr(pSTC_VTG_ascii->COURSEDEG_M,pNMEA);
        break;
        case POS_VTG_SPEEDKNOTS:
          strextr(pSTC_VTG_ascii->SPEEDKNOTS,pNMEA);
        break;
        case POS_VTG_SPEEDKMH:
          strextr(pSTC_VTG_ascii->SPEEDKMH,pNMEA);
        break;
        default:
        break;
      }        
      pNMEA=strextr(acBuffer,pNMEA);
      format_cnt++;
    } while (*pNMEA!=C_NMEA_CKS_EXPECT_CHAR&&(format_cnt<C_NMEA_MAXPACKETSIZE));
    if (format_cnt>=C_NMEA_MAXPACKETSIZE) return NULL;
    //get checksum at the end of transmission:
    strncpy_t(pSTC_VTG_ascii->CKS,++pNMEA,sizeof(pSTC_VTG_ascii->CKS)-1);
return pNMEA;
}

//--------------------------------------------------------------------------
//function	: GetGSAdataSTC()
//out		: NULL
//description	: Copies parts of GSA_Data from RAW_ASCII to formatted STC_ASCII
//--------------------------------------------------------------------------
extern char* GetGSAdataSTC(char* pNMEA)
{
char acBuffer[C_NMEA_MAXSTRSIZE];
uint8 format_cnt;

format_cnt=0;
    do
    {
      switch (format_cnt)
      {
        case POS_GSA_GPGSA:
        break;
        case POS_GSA_AUTODEF:
          pSTC_GSA_ascii->AUTODEF=*pNMEA;
        break;
        case POS_GSA_POSDEF:
          pSTC_GSA_ascii->POSDEF=*pNMEA;
        break;
        default:
        break;
      }        
      pNMEA=strextr(acBuffer,pNMEA);
      format_cnt++;
    } while (*pNMEA!=C_NMEA_CKS_EXPECT_CHAR&&(format_cnt<C_NMEA_MAXPACKETSIZE));
    if (format_cnt>=C_NMEA_MAXPACKETSIZE) return NULL;
    //get checksum at the end of transmission:
    strncpy_t(pSTC_GSA_ascii->CKS,++pNMEA,sizeof(pSTC_GSA_ascii->CKS)-1);
return pNMEA;
}

//--------------------------------------------------------------------------
//function	: NMEA_Acquire()
//out		: rc returns 1 in case of error while scanning NMEA strings
//description	: acquires and services all applied NMEA Strings
//                to ASCII and phys format
//--------------------------------------------------------------------------
int NMEA_Acquire(void)
{
static char *ptrData1;
char acBuffer[C_NMEA_MAXSTRSIZE];
int rc;

  rc=0;
  ptrData1=acBuffer;
  //get specific NMEA values and fill struct memory:
  if(GetRMCdataSTC(pSTC_NMEABuf_RAW->RMC)==NULL) rc|=1;
  if(GetGGAdataSTC(pSTC_NMEABuf_RAW->GGA)==NULL) rc|=1;
  if(GetVTGdataSTC(pSTC_NMEABuf_RAW->VTG)==NULL) rc|=1;
  if(GetGSAdataSTC(pSTC_NMEABuf_RAW->GSA)==NULL) rc|=1;
    //convert ASCII Data into physical values:
    pSTC_RMC_phys->DAT_year=atoi(strncpy_t(ptrData1,pSTC_RMC_ascii->DATE,2));
    pSTC_RMC_phys->DAT_month=atoi(strncpy_t(ptrData1,pSTC_RMC_ascii->DATE+2,2));
    pSTC_RMC_phys->DAT_day=atoi(strncpy_t(ptrData1,pSTC_RMC_ascii->DATE+4,2));
    pSTC_RMC_phys->TIME_h=atoi(strncpy_t(ptrData1,pSTC_RMC_ascii->TIME,2));
    pSTC_RMC_phys->TIME_min=atoi(strncpy_t(ptrData1,pSTC_RMC_ascii->TIME+2,2));
    pSTC_RMC_phys->TIME_sec=atoi(strncpy_t(ptrData1,pSTC_RMC_ascii->TIME+4,2));
    pSTC_GGA_phys->HEIGHT=RoundSTR_FPtoInt(pSTC_GGA_ascii->HEIGHT);
    pSTC_VTG_phys->COURSEDEG_T=RoundSTR_FPtoInt(pSTC_VTG_ascii->COURSEDEG_T);
    pSTC_VTG_phys->SPEEDKMH=RoundSTR_FPtoInt(pSTC_VTG_ascii->SPEEDKMH);
 
return rc;
}



