/*************************************************************************
 *
 *    Used with ICCARM and AARM.
 *
 *    File name   : nmea.h
 *    Description : NMEA Function Library Header
 *
 *    History :
 *    1. Date        : Feb, 2010
 *       Author      : Kampf Peter
 *       Description : Create
 *
 *    $Revision: 1
 **************************************************************************/
#ifndef __NMEA_H__
#define __NMEA_H__

#include "types.h"

#define   C_NMEA_DESIG_CHAR         '$'
#define   C_NMEA_CKS_EXPECT_CHAR    '*'
#define   C_NMEA_SEPARATION_CHAR    ','
#define   C_NMEA_DESIG_LENGTH       5
#define   C_NMEA_DATASTART          7

#define   C_NMEA_MAXSTRSIZE            128   //Max applied string Size
#define   C_NMEA_MAXPACKETSIZE         20    //Max applied packets in NMEAstring
#define   C_NMEA_TEMPBUFFERSIZE        30    //Max num chars in tempBuf

//for UART RX states:
#define   C_NMEA_IDLE                  0x0000
#define   C_NMEA_SYNC                  0x0001
#define   C_NMEA_CATCH_PREFIX          0x0002
//for UART frame flags:
#define   C_RMC_RCVD                   0x0001
#define   C_GGA_RCVD                   0x0002
#define   C_VTG_RCVD                   0x0004
#define   C_GSA_RCVD                   0x0008

#define   C_NMEA_GPGGA                      "GPGGA"
#define   C_NMEA_GPRMC                      "GPRMC"
#define   C_NMEA_GPVTG                      "GPVTG"
#define   C_NMEA_GPGSA                      "GPGSA"

//Prototypes private:

// ************** NMEA RAW STRUCT:
struct RAW_RXBuf {
char RMC[C_NMEA_MAXSTRSIZE];
char GGA[C_NMEA_MAXSTRSIZE];
char VTG[C_NMEA_MAXSTRSIZE];
char GSA[C_NMEA_MAXSTRSIZE];
};
typedef struct RAW_RXBuf STC_NMEA_Buf_RAW;

// ************** RMC STRUCTS:
struct RMCdata_phys {
uint8 DAT_year;
uint8 DAT_month;
uint8 DAT_day;
uint8 TIME_h;
uint8 TIME_min;
uint8 TIME_sec;
};
typedef struct RMCdata_phys STC_NMEA_RMC_phys;

struct RMCdata_ascii {
char TIME[10+1];
char STATUS;
char LATVAL[10+1];
char LATPREFIX;
char LONGVAL[10+1];
char LONGPREFIX;
char SPEEDKNOTS[5+1];
char COURSEDEG[7+1];
char DATE[6+1];
char MISLED[6+1];
char DEFMODE1;
char DEFMODE2;
char CKS[2+1];
};
typedef struct RMCdata_ascii STC_NMEA_RMC_ascii;

// ************** GGA STRUCTS:
struct GGAdata_ascii {
char QUALITY;
char SATELLITES[2+1];
char HDOP[4+1];
char HEIGHT[7+1];
char CKS[2+1];
};
typedef struct GGAdata_ascii STC_NMEA_GGA_ascii;

struct GGAdata_phys {
uint16 HEIGHT;
};
typedef struct GGAdata_phys STC_NMEA_GGA_phys;

// ************** VTG STRUCTS:
struct VTGdata_ascii {
char COURSEDEG_T[7+1];
char COURSEDEG_M[7+1];
char SPEEDKNOTS[7+1];
char SPEEDKMH[7+1];
char CKS[2+1];
};
typedef struct VTGdata_ascii STC_NMEA_VTG_ascii;

struct VTGdata_phys {
uint16 COURSEDEG_T;
uint16 SPEEDKNOTS;
uint16 SPEEDKMH;
};
typedef struct VTGdata_phys STC_NMEA_VTG_phys;

// ************** GSA STRUCTS:
struct GSAdata_ascii {
char AUTODEF;
char POSDEF;
char CKS[2+1];
};
typedef struct GSAdata_ascii STC_NMEA_GSA_ascii;

    
enum NMEA_RMC_Packet_Order
{
POS_RMC_GPRMC=0,
POS_RMC_TIME,
POS_RMC_STATUS,
POS_RMC_LATTITUDE,
POS_RMC_LATPREFIX,
POS_RMC_LONGITUDE,
POS_RMC_LONGPREFIX,
POS_RMC_SPEEDKNOTS,
POS_RMC_COURSEDEG,
POS_RMC_DATE,
POS_RMC_MISLED,
POS_RMC_DEFMODE1,
POS_RMC_DEFMODE2,
};

enum NMEA_GGA_Packet_Order
{
POS_GGA_GPGGA=0,
POS_GGA_TIME,
POS_GGA_LATTITUDE,
POS_GGA_LATPREFIX,
POS_GGA_LONGITUDE,
POS_GGA_LONGPREFIX,
POS_GGA_QUALITY,
POS_GGA_SATELLITES,
POS_GGA_HDOP,
POS_GGA_HEIGHT,
POS_GGA_HEIGHT_METRIC,
POS_GGA_GEO_ELLIPSE,
POS_GGA_GEO_ELLIPSE_METRIC,
};

enum NMEA_VTG_Packet_Order
{
POS_VTG_GPVTG=0,
POS_VTG_COURSEDEG_T,
POS_VTG_COURSEDEG_T_METRIC,
POS_VTG_COURSEDEG_M,
POS_VTG_COURSEDEG_M_METRIC,
POS_VTG_SPEEDKNOTS,
POS_VTG_SPEEDKNOTS_METRIC,
POS_VTG_SPEEDKMH,
POS_VTG_SPEEDKMH_METRIC,
};

enum NMEA_GSA_Packet_Order
{
POS_GSA_GPGSA=0,
POS_GSA_AUTODEF,
POS_GSA_POSDEF,
};

typedef enum {False, True}  boolean;

/* define all states in process: */
typedef enum {
    GPSNotAvailable = 0,
    GPSInit,
    GPSNotValid,
    GPSRMCValid,
    GPSValid,
} GPSstate;

//Prototypes pubilc:
extern char* GetRMCdataSTC(char* pNMEA);
extern char* GetGGAdataSTC(char* pNMEA);
extern char* GetVTGdataSTC(char* pNMEA);
extern char* GetGSAdataSTC(char* pNMEA);
extern int   NMEA_Acquire(void);

//public variables from module [nmea]:
extern STC_NMEA_Buf_RAW *pSTC_NMEABuf_RAW; 
extern STC_NMEA_RMC_ascii *pSTC_RMC_ascii;
extern STC_NMEA_GGA_ascii *pSTC_GGA_ascii;
extern STC_NMEA_VTG_ascii *pSTC_VTG_ascii;
extern STC_NMEA_GSA_ascii *pSTC_GSA_ascii;
extern STC_NMEA_RMC_phys *pSTC_RMC_phys;
extern STC_NMEA_GGA_phys *pSTC_GGA_phys;
extern STC_NMEA_VTG_phys *pSTC_VTG_phys;

#endif