/*************************************************************************
 *
 *    Used with ICCARM and AARM.
 *
 *    File name   : scheduler.c
 *    Description : State machine and OS of system
 *
 *    History :
 *    1. Date        : Feb, 2010
 *       Author      : Peter Kampf
 *
 *    $Revision: 14946 $
 **************************************************************************/
#include <stdio.h>
#include "system.h"
#include "hardware.h"
#include "utils.h"
#include "nmea.h"
#include "OLED_lib.h"
#include "timer.h"
#include "scheduler.h"

//define global variables for module scheduler.c:
volatile uint16 g_wdAPPSTAT=IDLEapp;
volatile uint16 g_wdSYSSTAT=IDLEsys;
volatile uint16 g_wdBitEvent;
volatile uint16 g_wdCount1=0;
volatile uint16 g_wdCount2=0;
//-----------------------------------------------
//function      : sm_Process/STATEMACHINE PROCESS
//out		: NULL
//description a): serializes events into parallel event-byte
//description b): dispatches events according to current state:
//-----------------------------------------------
extern void sm_process(void)
{
uint16 wdEvent=C_BIT_EV_NULL;
uint16 wdMemory, wdCounter;

  /* Scannen des Bytes auf gesetzte Bits */
  wdCounter=0;
  do
   {
    wdMemory=1<<(wdCounter); /* Maske zum erkennen von Bits erzeugen */
    if((wdMemory&g_wdBitEvent)!=0)
    {
       /* Gesetztes Bit ist erkannt, Ereignis extrahieren */
       wdEvent=wdMemory;
       /* Verarbeitetes Bit lschen */
       g_wdBitEvent &=~wdMemory;
    }
    wdCounter++;
   }while (
             (wdCounter<(sizeof(g_wdBitEvent)*16))&& //search for max. 16 times (bits)
             (wdEvent==C_BIT_EV_NULL) //if no bitevent found, shift next!
          );

//***********************************************
// System Dispatcher:
//***********************************************

//-----------------------------------------------
// Service [STATE:STARTUP]
//-----------------------------------------------
   	switch (g_wdSYSSTAT)
   	{
    	case STARTUP:
      	  switch (wdEvent)
      	  {
  		case C_BIT_EV_STARTUP: 
 	 		sm_Exec_Startup();
 		break;
  		case C_BIT_EV_GOTO_IDLE: 
 	 		sm_Exec_Idle();
                break;
                default:
                break; 
          }
    	break;
	}

//-----------------------------------------------
// Service [STATE:IDLE]
//-----------------------------------------------
   	switch (g_wdSYSSTAT)
   	{
    	case IDLEsys:
      	  switch (wdEvent)
      	  {
                case C_BIT_EV_ALIVE_TICK: 
 	 		sm_Exec_AliveThread();
 		break;
  		case C_BIT_EV_STARTUP: 
 	 		sm_Exec_Startup();
 		break;
  		case C_BIT_EV_BUTTON_APP: 
 	 		sm_Exec_AppChange();
                break;
  		case C_BIT_EV_NMEA_AVAIL: 
 	 		sm_Exec_NMEA_Callback();
 		break;
                case C_BIT_EV_GOTO_NODATA: 
 	 		sm_Exec_NODATA();
 		break;
  		case C_BIT_EV_GOTO_GPS_NOK: 
 	 		sm_Exec_GPS_NOK();
 		break;
  		case C_BIT_EV_GOTO_GPS_OK: 
 	 		sm_Exec_GPS_OK();
 		break;
                default:
                break; 
          }
    	break;
	}

//-----------------------------------------------
// Service [STATE:NO_DATA]
//-----------------------------------------------
   	switch (g_wdSYSSTAT)
   	{
    	case NO_DATA:
      	  switch (wdEvent)
      	  {
                case C_BIT_EV_ALIVE_TICK: 
 	 		sm_Exec_AliveThread();
 		break;
                case C_BIT_EV_TOGCOUNTER1: 
 	 		sm_Exec_TogThread1();
 		break;
                case C_BIT_EV_NMEA_AVAIL: 
 	 		sm_Exec_NMEA_Callback();
 		break;
  		case C_BIT_EV_GOTO_GPS_NOK: 
 	 		sm_Exec_GPS_NOK();
 		break;
  		case C_BIT_EV_GOTO_GPS_OK: 
 	 		sm_Exec_GPS_NOK();
 		break;
                default:
                break; 
          }
    	break;
	}

//-----------------------------------------------
// Service [STATE:GPS_NOK]
//-----------------------------------------------
   	switch (g_wdSYSSTAT)
   	{
    	case GPS_NOK:
      	  switch (wdEvent)
      	  {
                case C_BIT_EV_ALIVE_TICK: 
 	 		sm_Exec_AliveThread();
 		break;
  		case C_BIT_EV_BUTTON_APP: 
 	 		sm_Exec_AppChange();
                break;
                case C_BIT_EV_TOGCOUNTER1: 
 	 		sm_Exec_TogThread1();
 		break;
  		case C_BIT_EV_NMEA_AVAIL: 
 	 		sm_Exec_NMEA_Callback();
 		break;
  		case C_BIT_EV_GOTO_GPS_OK: 
 	 		sm_Exec_GPS_OK();
 		break;
  		case C_BIT_EV_GOTO_NODATA: 
 	 		sm_Exec_NODATA();
 		break;
                default:
                break; 
          }
    	break;
	}

//-----------------------------------------------
// Service [STATE:GPS_OK]
//-----------------------------------------------
   	switch (g_wdSYSSTAT)
   	{
    	case GPS_OK:
      	  switch (wdEvent)
      	  {
                case C_BIT_EV_ALIVE_TICK: 
 	 		sm_Exec_AliveThread();
 		break;
  		case C_BIT_EV_BUTTON_APP: 
 	 		sm_Exec_AppChange();
                break;
  		case C_BIT_EV_NMEA_AVAIL: 
 	 		sm_Exec_NMEA_Callback();
 		break;
  		case C_BIT_EV_GOTO_GPS_NOK: 
 	 		sm_Exec_GPS_NOK();
 		break;
  		case C_BIT_EV_GOTO_NODATA: 
 	 		sm_Exec_NODATA();
 		break;
                default:
                break; 
          }
    	break;
	}
}

//----------------------------------------------
//----------------------------------------------
// \/\/\/\/\/\/\ EXT EVENT THREADS: /\/\/\/\/\/\/
//----------------------------------------------
//----------------------------------------------

//----------------------------------------------
//function	: sm_Exec_NMEA_Callback()
//out		: NULL
//description	: StateM Executable: Startup Event
//-----------------------------------------------
static void sm_Exec_NMEA_Callback(void)
{
  NMEA_Acquire();
  if(pSTC_RMC_ascii->STATUS=='A')
  {
    //set GPSstate according to GPS reception:
    g_wdBitEvent|=C_BIT_EV_GOTO_GPS_OK;
    //set/refresh GPSNOK timer:
    TMR_Manager(C_TMR_GPSNOKtimeout,C_TMR_ON);
  }
  else g_wdBitEvent|=C_BIT_EV_GOTO_GPS_NOK;
  //set/refresh NO Data timer only in state !GPS_NOK
  //but show it staticly if LastApplication is selected:
  if(g_wdSYSSTAT!=GPS_NOK) OLED_PrintApplicationDyn();
  else if(g_wdAPPSTAT==AppX) OLED_PrintApplicationDyn();
  TMR_Manager(C_TMR_NoDATAtimeout,C_TMR_ON);
}
//----------------------------------------------
//function	: sm_Exec_NODATA()
//out		: NULL
//description	: StateM Executable: Startup Event
//-----------------------------------------------
static void sm_Exec_NODATA(void)
{
  OLED_PrintNoDataScreen();
  g_wdSYSSTAT=NO_DATA;
}
//----------------------------------------------
//function	: sm_Exec_GPS_NOK()
//out		: NULL
//description	: StateM Executable: Startup Event
//-----------------------------------------------
static void sm_Exec_GPS_NOK(void)
{
//  OLED_PrintNoGPSScreen();
    OLED_PrintStaticMask();
    g_wdSYSSTAT=GPS_NOK;
}
//----------------------------------------------
//function	: sm_Exec_GPS_OK()
//out		: NULL
//description	: StateM Executable: Startup Event
//-----------------------------------------------
static void sm_Exec_GPS_OK(void)
{
  OLED_PrintStaticMask();
  g_HeightMax=0;      //reset MaxHeight
  g_wdSYSSTAT=GPS_OK;
}
//----------------------------------------------
//function	: sm_Exec_Startup()
//out		: NULL
//description	: StateM Executable: Startup Event
//-----------------------------------------------
static void sm_Exec_Startup(void)
{
    OLED_PrintStartupScreen();
    TMR_Manager(C_TMR_StartupTimeout,C_TMR_ON);
}
//----------------------------------------------
//function	: sm_Exec_Idle()
//out		: NULL
//description	: StateM Executable: Idle Event
//-----------------------------------------------
static void sm_Exec_Idle(void)
{
    TMR_Manager(C_TMR_NoDATAtimeout,C_TMR_ON);
    TMR_Manager(C_TMR_GPSNOKtimeout,C_TMR_ON);
    g_wdSYSSTAT=IDLEsys;
}
//----------------------------------------------
//function	: sm_Exec_AliveThread()
//out		: NULL
//description	: StateM Executable:
//-----------------------------------------------
static void sm_Exec_AliveThread(void)
{
static volatile uint8 s_bMerker;
  XORLED1(s_bMerker);
}
//----------------------------------------------
//function	: smExec_DMX_ERR()
//out		: NULL
//description	: StateM Executable:[DMXadd_NOK Event]
//-----------------------------------------------
static void sm_Exec_AppChange(void)
{
  //switch to next application:
  if(++g_wdAPPSTAT>AppX) g_wdAPPSTAT=App1;
  g_strlengthMem0=0;
  g_strlengthMem1=0;
  OLED_PrintStaticMask();
}

//----------------------------------------------
//----------------------------------------------
/* /\/\/\/\/\/\ PERIODICAL THREADS: /\/\/\/\/ */
//----------------------------------------------
//----------------------------------------------

//----------------------------------------------
//function	: sm_Exec_TogThread1()
//out		: NULL
//description	: StateM Executable:
//-----------------------------------------------

static void sm_Exec_TogThread1(void)
{
static volatile uint8 s_bMerker;

  switch (g_wdSYSSTAT)
  {
    	case GPS_NOK:
          if(g_wdAPPSTAT!=AppX)
          {
            if(s_bMerker) {cls_y(20,60,0); cls_y(20,60,1);}
            else OLED_PrintApplicationDyn();
          }
        break;
    	case NO_DATA:
          if(s_bMerker) {cls_y(20,60,0); cls_y(20,60,1);}
          else OLED_PrintNoDataScreen();
        break;
  }
  s_bMerker^=1;
  
}

