/*************************************************************************
 *
 *    Used with ICCARM and AARM.
 *
 *    File name   : system.c
 *    Description : System specific routines
 *
 *    History :
 *    1. Date        : Jan, 2010
 *       Author      : Peter Kampf
 *
 *    $Revision: 14946 $
 **************************************************************************/
#include <stdbool.h>
#include "hardware.h"
#include "system.h"
#include "uart.h"

#define   ROM_RUN // Interrupt Vectors in FLASH

/******************************************************************************
 * Function Name: Global_SystemInit()
 *
 * Description:
 *    This function is responsible for initializing the program
 *    specific hardware
 * Calling Sequence: 
 *    void
 * Returns:
 *    void
 *****************************************************************************/
void uC_Init(void)
{
  lowLevelInit();                       // essential setup PLL & processor port pins

#if defined(RAM_RUN)
  MEMMAP = MEMMAP_SRAM;                 // map interrupt vectors space into SRAM
#elif defined(ROM_RUN)
  MEMMAP = MEMMAP_FLASH;                // map interrupt vectors space into FLASH
#else
#error RUN_MODE not defined!
#endif

  VICIntEnClear = 0xFFFFFFFF;           // clear all interrupts
  VICIntSelect = 0x00000000;            // clear all FIQ selections

  /* Init TIMER0 */
  timer0_ISRVEC_init();
  timer0_init_start();
  
  /* Init UART1 */
  UART1_Init(UART_BAUD(HOST_BAUD), UART_8N1, UART_FIFO_OFF); // setup the UART1;
  
  //SPECIFIC definition of direction for LED port: (port0.12 & port0.13):
  IODIR  |= GPIO_OUTPUT_SET_MASK;
  //SPECIFIC definition of PIN Function Select Register0:
  PINSEL0 |= GPIO_PINSEL_MASK;
  
  /* SPI Stuff */
  //DEVICE set to highest speed for SPI at 10MHz -> 1.25MHz
  S0SPCCR= 0x08;
  //DEVICE selected as master
  S0SPCR= 0x20;
}

/******************************************************************************
 * Function Name: Dly100us()
 *
 * Description:
 *    Delay [100us Steps]
 *    Return: void
 *
 * Calling Sequence: 
 *    void *arg
 *
 * Returns:
 *    void
 *****************************************************************************/
void delay_ms(long arg)
{
volatile uint32 Dly = (uint32)arg;
volatile uint16 Dly100;
    for(;Dly;Dly--) {for(Dly100=0x348; Dly100; Dly100--);}
}

/******************************************************************************
 * Function Name: Delay()
 *
 * Description:
 *    Delay
 *    Return: void
 *
 * Calling Sequence: 
 *    void uint32_t
 *
 * Returns:
 *    void
 *****************************************************************************/
void Delay(uint32 del)
{
  while (--del!=0);
}

/******************************************************************************
 * Function Name: global IRQ_Handler()
 *
 * Description:
 *    Delay
 *    Return: void
 *
 * Calling Sequence: 
 *
 * Returns:
 *    void
 *****************************************************************************/
__irq __arm void IRQ_Handler(void)
{
  void (*interrupt_function)(void);
  uint16 vector;

  vector = VICVectAddr;                     // GET VECTOR ADDRESS FROM VIC CONTROLLER
  interrupt_function = (void(*)())vector;

  (*interrupt_function)();                  // Call vectored interrupt function.

  VICVectAddr = 0;
}
/*-----------------------------------------------------------------------------*/

