/*************************************************************************
 *
 *    Used with ICCARM and AARM.
 *
 *    File name   : system.h
 *    Description : System definitions
 *
 *    History :
 *    1. Date        : Jan, 2010
 *       Author      : Kampf Peter
 *       Description : Create
 *
 *    $Revision: 1
 **************************************************************************/
#ifndef __SYSTEM_H__
#define __SYSTEM_H__

#include <intrinsics.h>
#include "types.h"

/*-------------------------------------------------------------------------------------------------------------------------------*/
#define C_HEADERINFO_TITLE        "GPS TELEMETER"
#define C_HEADERINFO_AUTHOR       "P.KAMPF/2010"
#define C_HEADERINFO_SWRELEASE    "v1.03/27.02"

#define GPIO_OUTPUT_SET_MASK    0xFFFEFDFF               
#define GPIO_PINSEL_MASK        0x00051500 // enable UART1:TXD/RXD, enable SPI

//specific substitutes:
#define EN_Interrupt            __enable_interrupt();
#define DIS_Interrupt           __disable_interrupt();
  
// Declarations of functions coded in system.c
__irq __arm void IRQ_Handler(void);

//prototypes for declaration used externally by including system.h:
void uC_Init(void);
void Delay(uint32 del);
void delay_ms(long);

/* SystemStates:
STARTUP:  System State at Startup
IDLEsys:  System IdleState after startup
NO_DATA:  No Serial Data or NMEA available
GPS_NOK:  GPS Data available but not valid
GPS_OK:   GPS Data available and valid
*/
enum SystemStates {STARTUP=0, IDLEsys, NO_DATA, GPS_NOK, GPS_OK};
/* Application States:
IDLEapp:  Application IdleState at startup
App1:     Application 1: Speed, Heading and Height+Height Max.
App2:     Application 2: Speed, Heading
AppX:     Application X: Raw Data: Time, Date, GPSAttributes
*/
enum ApplicationStates {IDLEapp=0, App1, App2, AppX};

//external prototypes:
extern void UART1_Init(uint16, uint8, uint8);
extern void timer0_ISRVEC_init(void);
extern void timer0_init_start(void);
          
//UART define BaudRate:
#define HOST_BAUD           (38400)

//Definition of Events:
#define C_BIT_EV_NULL			0x0000  //->Zero Event
#define C_BIT_EV_BUTTON_APP   	        0x0001	//->Application Event Bit
#define C_BIT_EV_NMEA_AVAIL   	        0x0002	//->NMEA Available Event Bit
#define C_BIT_EV_ALIVE_TICK             0x0004
#define C_BIT_EV_GOTO_NODATA	        0x0008	//->event if there is no NMEA Data
#define C_BIT_EV_GOTO_GPS_NOK	        0x0010	//->event if there is invalid NMEA Data
#define C_BIT_EV_GOTO_GPS_OK	        0x0020	//->event if there is valid NMEA Data
#define C_BIT_EV_GOTO_IDLE              0x0040  //->event after startup has finished
#define C_BIT_EV_NMI_TICK_THREAD0   	0x0080	//->Thread0 Event Bit

#define C_BIT_EV_NMI_TICK_THREAD1   	0x0100	//->Thread1 Event Bit
#define C_BIT_EV_TOGCOUNTER1            0x0200
#define C_BIT_EV_STARTUP                0x0400

/* Constants */
#define C_PREVAL_THREADSPEED0   	10   //Global divisor for TMR0 building tick (100ms)
#define C_PREVAL_THREADSPEED1   	40
#define C_PREVAL_THREADSPEED2   	40
#define C_PREVAL_TO_NODATA_EXPIRY   	600
#define C_PREVAL_TO_GPSNOK_EXPIRY   	500
#define C_PREVAL_TO_STARTUP_EXPIRY   	200

#define C_PREVAL_DEBOUNCEVAL0           5   //number of get_switch times => (5*10ms)

//#define C_TMR_NoGPStimeout

//Timer:
#define C_TMR_ON			FALSE
#define C_TMR_OFF			TRUE

//SoftTimernames:
#define	C_TMR_NoDATAtimeout		0		//State Fallback TO to state: NoData
#define	C_TMR_GPSNOKtimeout		1		//State Fallback TO to state: GPSNOK
#define	C_TMR_StartupTimeout		2		//State Fallback TO to state: Idle

#endif /* __SYSTEM_H__ */