/*************************************************************************
 *
 *    Used with ICCARM and AARM.
 *
 *    File name   : utils.c
 *    Description : Utilities/Tools
 *
 *    History :
 *    1. Date        : Jan, 2010
 *       Author      : Peter Kampf
 *
 *    $Revision: 1 $
 **************************************************************************/
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include "types.h"
#include "hardware.h"
#include "system.h"
#include "utils.h"
#include "nmea.h"

//----------------------------------------------
//function	: strncpy_t
//out		: NULL
//description	: strncpy with Null Termination @end
//returns       : pointer to copied destination
//----------------------------------------------
extern char* strncpy_t(char* ptrDest, char* ptrSrc, uint8 num_chars)
{
char* ptrTmp;
    strncpy(ptrDest,ptrSrc,num_chars);
    ptrTmp=ptrDest;
    ptrTmp+=num_chars;
    *ptrTmp = '\0';
return ptrDest;    
}

//----------------------------------------------
//function	: strncpy_p
//out		: NULL
//description	: strncpy with Null Termination @end
//returns       : pointer to copied destination
//----------------------------------------------
extern char* strncpy_p(char* ptrDest, const char* ptrSrc, uint8 num_chars)
{
char* ptrTmp;
    strncpy(ptrDest,ptrSrc,num_chars);
    ptrTmp=ptrDest;
    ptrTmp+=num_chars;
    *ptrTmp = '\0';
return ptrDest;    
}

//----------------------------------------------
//function	: strextr
//out		: NULL
//description	: extracts stringportion until next sep.char
//returns       : pointer to next destination
//----------------------------------------------
extern char *strextr(char* ptrDest, char* ptrSrc)
{
static uint8 maxcnt;
  maxcnt=0;
  while((*ptrSrc!=C_NMEA_SEPARATION_CHAR)
                  &&(*ptrSrc!=C_NMEA_CKS_EXPECT_CHAR)
                  &&(maxcnt++<C_NMEA_MAXSTRSIZE))
  *ptrDest++=*ptrSrc++; 
  *ptrDest='\0';
  if(maxcnt>=C_NMEA_MAXSTRSIZE) 
  {
    return NULL; //check overflow
  }
  if(*ptrSrc==C_NMEA_CKS_EXPECT_CHAR) return ptrSrc;  //check asterix
return ++ptrSrc; //return next token position
}

//--------------------------------------------------------------------------
//function	: RoundSTR_FPtoInt()
//out		: returns Int Value
//description	: rounds Point Format in String to an Int
//--------------------------------------------------------------------------
extern int RoundSTR_FPtoInt(char* ptrSrc)
{
char acBuffer[C_NMEA_TEMPBUFFERSIZE];
char *ptrDest, *ptrTemp;
uint16 result, offset, diff;

  ptrDest=acBuffer;
  while(*ptrSrc!='\0') //eliminate point:  
  {
      if(*ptrSrc!='.') *ptrDest++=*ptrSrc++;
      else ptrTemp=++ptrSrc; //save pos for ptr.difference calc
  }
  *ptrDest='\0';
  ptrDest=acBuffer;
  diff= ptrSrc-ptrTemp; // calculate size of floating
  //round to next digit div by 10, i.e. 3973 => 398
  result=atoi(ptrDest);
  //calculate rounding method according to floating size:
  if (1==diff) offset=1; else offset=10;
  result +=5*offset; 
  result /=10*offset;
return result;
}
